#include "Parameters.h"
#include "Pedigree.h"
#include "math.h"
#include <iostream>
#include "FastFit.h"
#include "TraitTransformations.h"
#include "TransformResiduals.h"
#include <stdio.h>
#include <time.h>
#include "Error.h"
#include "AutoFit.h"
#include "PreMeta.h"
#include "OutputKin.h"

int main(int argc, char ** argv)
{

   printf("\nRARE-METAL-WORKER -- A Forerunner of Rare-Metal\n"
          "          (c) 2012-2013 Shuang Feng, Goncalo Abecasis\n\n");

   clock_t start = clock();
   String pedfile, datfile;
   double tol = 0.0001;

   BEGIN_LONG_PARAMETERS(additional)
      LONG_PARAMETER_GROUP("Input Files")
      LONG_STRINGPARAMETER("ped", &pedfile)
      LONG_STRINGPARAMETER("dat", &datfile)
      LONG_STRINGPARAMETER("vcf", &PreMeta::vcfInput)
      LONG_PARAMETER_GROUP("Output Files")
      LONG_STRINGPARAMETER("prefix", &PreMeta::outputFile)
      LONG_INTPARAMETER("LDwindow", &PreMeta::window)
      LONG_PARAMETER_GROUP("VC Options")
      LONG_PARAMETER("vcShared", &AutoFit::fitSharedEnvironment)
      LONG_PARAMETER("vcX", &AutoFit::fitX)
      LONG_PARAMETER("useCovariates", &FastFit::useCovariates)
      LONG_PARAMETER_GROUP("Trait Options")
      LONG_PARAMETER("makeResiduals", &FastFit::regressOutCov)
      LONG_PARAMETER("inverseNormal", &FastFit::inverseNormal)
      LONG_STRINGPARAMETER("traitName", &FastFit::traitName)
      LONG_PARAMETER_GROUP("Kinship Source")
      LONG_PARAMETER("kinPedigree", &FastTransform::pedKin)
      LONG_PARAMETER("kinGeno", &FastTransform::empKin)
      LONG_STRINGPARAMETER("kinFile", &FastTransform::readInEmp)
      LONG_PARAMETER("kinSave", &OutputKin::outputKin)
      LONG_PARAMETER_GROUP("Kinship Options")
      LONG_DOUBLEPARAMETER("kinMaf", &KinshipEmp::q)
      LONG_DOUBLEPARAMETER("kinMiss", &KinshipEmp::miss)
      LONG_PARAMETER_GROUP("Chromosome X")
      LONG_STRINGPARAMETER("xLabel", &PreMeta::xLabel)
      LONG_INTPARAMETER("xStart", &PreMeta::Xstart)
      LONG_INTPARAMETER("xEnd", &PreMeta::Xend)

      END_LONG_PARAMETERS();

   ParameterList pl;
   pl.Add(new LongParameters("Options:", additional));

   pl.Read(argc, argv);
   pl.Status();

   Pedigree ped;
   ped.Prepare(datfile);
   ped.Load(pedfile);

   clock_t readin = clock();
   printf("Reading in genotype files used %f seconds.\n", ((double)readin - start) / CLOCKS_PER_SEC);

   if(ped.familyCount == ped.count && !FastTransform::empKin && !FastTransform::pedKin)
   {
      FastFit::unrelated = true;
      printf("Now working on these individuals as if they are unrelated.\n");
   }

   if(ped.markerCount>0)
      PreMeta::genoFromPed = true;
   else
      PreMeta::genoFromVCF=true;

   if(PreMeta::dosage && !PreMeta::genoFromVCF)
   {
      error("ERROR! Dosage has to be from vcf file.\n");
   }
   AutoFit fitEngine;
   FastFit engine;
   FastTransform trans;

   StringArray traits;
   if(!FastFit::traitName.IsEmpty())
   {

      if(FastFit::traitName.Find(".txt",0) !=-1)
	 //if(FastFit::traitName == "traitName.txt")
      {
	 FILE * trFile = fopen(FastFit::traitName,"r");
	 String buffer;
	 while (!feof(trFile))
	 {
	    buffer.ReadLine(trFile);
	    traits.AddTokens(buffer, SEPARATORS);
	 }
	 fclose(trFile);
      }
      else
      {
	 traits.AddTokens(FastFit::traitName,"/");
      }
   }
   else 
   {
      traits = ped.traitNames;
   }

   for(int i=0;i<traits.Length();i++)
   {
      String traitName;
      traitName.Copy(traits[i]);
      engine.FitLinearModels(fitEngine,ped,tol,trans,traitName);

      clock_t fitModel = clock();
      printf("Decomposition and fitting linear model used %f seconds.\n", ((double)fitModel - (double)readin) / CLOCKS_PER_SEC);

      PreMeta preMeta;
      preMeta.Run(ped,trans,engine,traits[i]);

      clock_t analysis = clock();
      printf("Association tests used %f seconds.\n", ((double)analysis - (double)fitModel) / CLOCKS_PER_SEC);
   }

   return 0;
}
